@@ -7,7 +7,7 @@ from collections import namedtuple
 from typing import List
 
 import kubernetes
-from dagster import Array, Field, Noneable, StringSource
+from dagster import Array, BoolSource, Field, Noneable, StringSource
 from dagster import __version__ as dagster_version
 from dagster import check
 from dagster.config.field_utils import Permissive, Shape
@@ -16,6 +16,8 @@ from dagster.core.errors import DagsterInvalidConfigError
 from dagster.serdes import whitelist_for_serdes
 from dagster.utils import frozentags, merge_dicts
 
+from .models import k8s_model_from_dict
+
 # To retry step job, users should raise RetryRequested() so that the dagster system is aware of the
 # retry. As an example, see retry_pipeline in dagster_test.test_project.test_pipelines.repo
 # To override this config, user can specify UserDefinedDagsterK8sConfig.
@@ -183,7 +185,7 @@ class DagsterK8sJobConfig(
         "_K8sJobTaskConfig",
         "job_image dagster_home image_pull_policy image_pull_secrets service_account_name "
         "instance_config_map postgres_password_secret env_config_maps env_secrets env_vars "
-        "volume_mounts",
+        "volume_mounts volumes",
     )
 ):
     """Configuration parameters for launching Dagster Jobs on Kubernetes.
@@ -220,7 +222,11 @@ class DagsterK8sJobConfig(
             Default: ``[]``. See: https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
         job_image (Optional[str]): The docker image to use. The Job container will be launched with this
             image. Should not be specified if using userDeployments.
-
+        volume_mounts (Optional[List[Permissive]]): A list of volume mounts to include in the job's
+            container. Default: ``[]``. See:
+            https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volumemount-v1-core
+        volumes (Optional[List[Permissive]]): A list of volumes to include in the Job's Pod. Default: ``[]``. See:
+            https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volume-v1-core
     """
 
     def __new__(
@@ -236,6 +242,7 @@ class DagsterK8sJobConfig(
         env_secrets=None,
         env_vars=None,
         volume_mounts=None,
+        volumes=None,
     ):
         return super(DagsterK8sJobConfig, cls).__new__(
             cls,
@@ -256,6 +263,7 @@ class DagsterK8sJobConfig(
             env_secrets=check.opt_list_param(env_secrets, "env_secrets", of_type=str),
             env_vars=check.opt_list_param(env_vars, "env_secrets", of_type=str),
             volume_mounts=check.opt_list_param(volume_mounts, "volume_mounts"),
+            volumes=check.opt_list_param(volumes, "volumes"),
         )
 
     @classmethod
@@ -346,6 +354,38 @@ class DagsterK8sJobConfig(
                 "Default: ``[]``. See: "
                 "https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables",
             ),
+            "volume_mounts": Field(
+                Array(
+                    Shape(
+                        {
+                            "name": StringSource,
+                            "mountPath": StringSource,
+                            "mountPropagation": Field(StringSource, is_required=False),
+                            "readOnly": Field(BoolSource, is_required=False),
+                            "subPath": Field(StringSource, is_required=False),
+                            "subPathExpr": Field(StringSource, is_required=False),
+                        }
+                    )
+                ),
+                is_required=False,
+                default_value=[],
+                description="A list of volume mounts to include in the job's container. Default: ``[]``. See: "
+                "https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volumemount-v1-core",
+            ),
+            "volumes": Field(
+                Array(
+                    Permissive(
+                        {
+                            "name": str,
+                        }
+                    )
+                ),
+                is_required=False,
+                default_value=[],
+                description="A list of volumes to include in the Job's Pod. Default: ``[]``. For the many "
+                "possible volume source types that can be included, see: "
+                "https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volume-v1-core",
+            ),
         }
 
     @property
@@ -473,7 +513,7 @@ def construct_dagster_k8s_job(
     user_defined_k8s_env_vars = user_defined_k8s_config.container_config.pop("env", [])
     for env_var in user_defined_k8s_env_vars:
         additional_k8s_env_vars.append(
-            kubernetes.client.V1EnvVar(name=env_var["name"], value=env_var["value"])
+            k8s_model_from_dict(kubernetes.client.models.V1EnvVar, env_var)
         )
 
     user_defined_k8s_env_from = user_defined_k8s_config.container_config.pop("env_from", [])
@@ -503,11 +543,7 @@ def construct_dagster_k8s_job(
             sub_path="dagster.yaml",
         )
     ] + [
-        kubernetes.client.V1VolumeMount(
-            name=mount["name"],
-            mount_path=mount["path"],
-            sub_path=mount["sub_path"],
-        )
+        k8s_model_from_dict(kubernetes.client.models.V1VolumeMount, mount)
         for mount in job_config.volume_mounts
     ]
 
@@ -516,9 +552,10 @@ def construct_dagster_k8s_job(
     user_defined_k8s_volume_mounts = user_defined_k8s_config.container_config.pop(
         "volume_mounts", []
     )
-    additional_k8s_volume_mounts = []
     for volume_mount in user_defined_k8s_volume_mounts:
-        additional_k8s_volume_mounts.append(kubernetes.client.V1VolumeMount(**volume_mount))
+        volume_mounts.append(
+            k8s_model_from_dict(kubernetes.client.models.V1VolumeMount, volume_mount)
+        )
 
     job_container = kubernetes.client.V1Container(
         name=job_name,
@@ -527,10 +564,12 @@ def construct_dagster_k8s_job(
         image_pull_policy=job_config.image_pull_policy,
         env=env + job_config.env + additional_k8s_env_vars,
         env_from=job_config.env_from_sources + additional_k8s_env_from,
-        volume_mounts=volume_mounts + additional_k8s_volume_mounts,
+        volume_mounts=volume_mounts,
         **user_defined_k8s_config.container_config,
     )
 
+    user_defined_volumes = user_defined_k8s_config.pod_spec_config.pop("volumes", [])
+
     volumes = [
         kubernetes.client.V1Volume(
             name="dagster-instance",
@@ -538,19 +577,15 @@ def construct_dagster_k8s_job(
                 name=job_config.instance_config_map
             ),
         )
-    ] + [
-        kubernetes.client.V1Volume(
-            name=mount["name"],
-            config_map=kubernetes.client.V1ConfigMapVolumeSource(name=mount["configmap"]),
-        )
-        if mount.get("configmap")
-        else kubernetes.client.V1Volume(
-            name=mount["name"],
-            secret=kubernetes.client.V1SecretVolumeSource(secret_name=mount["secret"]),
-        )
-        for mount in job_config.volume_mounts
     ]
 
+    for volume in job_config.volumes + user_defined_volumes:
+        new_volume = k8s_model_from_dict(
+            kubernetes.client.models.V1Volume,
+            volume,
+        )
+        volumes.append(new_volume)
+
     # If the user has defined custom labels, remove them from the pod_template_spec_metadata
     # key and merge them with the dagster labels
     user_defined_pod_template_labels = user_defined_k8s_config.pod_template_spec_metadata.pop(
