@@ -4,9 +4,8 @@ import kubernetes
 from dagster import EventMetadataEntry, Field, Noneable, StringSource, check
 from dagster.cli.api import ExecuteRunArgs
 from dagster.core.events import EngineEventData
-from dagster.core.host_representation import ExternalPipeline
-from dagster.core.launcher import RunLauncher
-from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
+from dagster.core.launcher import LaunchRunContext, RunLauncher
+from dagster.core.storage.pipeline_run import PipelineRunStatus
 from dagster.core.storage.tags import DOCKER_IMAGE_TAG
 from dagster.serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
 from dagster.utils import frozentags, merge_dicts
@@ -22,9 +21,9 @@ from .utils import delete_job
 
 
 class K8sRunLauncher(RunLauncher, ConfigurableClass):
-    """RunLauncher that starts a Kubernetes Job for each pipeline run.
+    """RunLauncher that starts a Kubernetes Job for each Dagster job run.
 
-    Encapsulates each pipeline run in a separate, isolated invocation of ``dagster-graphql``.
+    Encapsulates each run in a separate, isolated invocation of ``dagster-graphql``.
 
     You may configure a Dagster instance to use this RunLauncher by adding a section to your
     ``dagster.yaml`` like the following:
@@ -35,7 +34,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             module: dagster_k8s.launcher
             class: K8sRunLauncher
             config:
-                service_account_name: pipeline_run_service_account
+                service_account_name: your_service_account
                 job_image: my_project/dagster_image:latest
                 instance_config_map: dagster-instance
                 postgres_password_secret: dagster-postgresql-secret
@@ -77,7 +76,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             and https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.17/#podspec-v1-core.
         image_pull_policy (Optional[str]): Allows the image pull policy to be overridden, e.g. to
             facilitate local testing with `kind <https://kind.sigs.k8s.io/>`_. Default:
-            ``"Always"``. See: https://kubernetes.io/docs/concepts/containers/images/#updating-images.
+            ``"IfNotPresent"``. See: https://kubernetes.io/docs/concepts/containers/images/#updating-images.
         job_namespace (Optional[str]): The namespace into which to launch new jobs. Note that any
             other Kubernetes resources the Job requires (such as the service account) must be
             present in this namespace. Default: ``"default"``
@@ -87,6 +86,13 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         env_secrets (Optional[List[str]]): A list of custom Secret names from which to
             draw environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:
             https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
+        env_vars (Optional[List[str]]): A list of environment variables to inject into the Job.
+            Default: ``[]``. See: https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
+        volume_mounts (Optional[List[Permissive]]): A list of volume mounts to include in the job's
+            container. Default: ``[]``. See:
+            https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volumemount-v1-core
+        volumes (Optional[List[Permissive]]): A list of volumes to include in the Job's Pod. Default: ``[]``. See:
+            https://v1-18.docs.kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#volume-v1-core
     """
 
     def __init__(
@@ -96,7 +102,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         postgres_password_secret=None,
         dagster_home=None,
         job_image=None,
-        image_pull_policy="Always",
+        image_pull_policy=None,
         image_pull_secrets=None,
         load_incluster_config=True,
         kubeconfig_file=None,
@@ -104,11 +110,16 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         job_namespace="default",
         env_config_maps=None,
         env_secrets=None,
+        env_vars=None,
         k8s_client_batch_api=None,
+        volume_mounts=None,
+        volumes=None,
     ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self.job_namespace = check.str_param(job_namespace, "job_namespace")
 
+        self.load_incluster_config = load_incluster_config
+        self.kubeconfig_file = kubeconfig_file
         if load_incluster_config:
             check.invariant(
                 kubeconfig_file is None,
@@ -124,7 +135,9 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         self._job_config = None
         self._job_image = check.opt_str_param(job_image, "job_image")
         self.dagster_home = check.str_param(dagster_home, "dagster_home")
-        self._image_pull_policy = check.str_param(image_pull_policy, "image_pull_policy")
+        self._image_pull_policy = check.opt_str_param(
+            image_pull_policy, "image_pull_policy", "IfNotPresent"
+        )
         self._image_pull_secrets = check.opt_list_param(
             image_pull_secrets, "image_pull_secrets", of_type=dict
         )
@@ -137,9 +150,40 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             env_config_maps, "env_config_maps", of_type=str
         )
         self._env_secrets = check.opt_list_param(env_secrets, "env_secrets", of_type=str)
+        self._env_vars = check.opt_list_param(env_vars, "env_vars", of_type=str)
+        self._volume_mounts = check.opt_list_param(volume_mounts, "volume_mounts")
+        self._volumes = check.opt_list_param(volumes, "volumes")
 
         super().__init__()
 
+    @property
+    def image_pull_policy(self):
+        return self._image_pull_policy
+
+    @property
+    def image_pull_secrets(self):
+        return self._image_pull_secrets
+
+    @property
+    def service_account_name(self):
+        return self._service_account_name
+
+    @property
+    def env_config_maps(self):
+        return self._env_config_maps
+
+    @property
+    def env_secrets(self):
+        return self._env_secrets
+
+    @property
+    def volume_mounts(self):
+        return self._volume_mounts
+
+    @property
+    def volumes(self):
+        return self._volumes
+
     @property
     def _batch_api(self):
         return self._fixed_batch_api if self._fixed_batch_api else kubernetes.client.BatchV1Api()
@@ -190,6 +234,9 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
                     self._env_config_maps, "env_config_maps", of_type=str
                 ),
                 env_secrets=check.opt_list_param(self._env_secrets, "env_secrets", of_type=str),
+                env_vars=check.opt_list_param(self._env_vars, "env_vars", of_type=str),
+                volume_mounts=self._volume_mounts,
+                volumes=self._volumes,
             )
             return self._job_config
 
@@ -212,18 +259,20 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
                 self._env_config_maps, "env_config_maps", of_type=str
             ),
             env_secrets=check.opt_list_param(self._env_secrets, "env_secrets", of_type=str),
+            env_vars=check.opt_list_param(self._env_vars, "env_vars", of_type=str),
+            volume_mounts=self._volume_mounts,
+            volumes=self._volumes,
         )
 
-    def launch_run(self, run, external_pipeline):
-        check.inst_param(run, "run", PipelineRun)
-        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+    def launch_run(self, context: LaunchRunContext) -> None:
+        run = context.pipeline_run
 
         job_name = "dagster-run-{}".format(run.run_id)
         pod_name = job_name
 
         user_defined_k8s_config = get_user_defined_k8s_config(frozentags(run.tags))
 
-        pipeline_origin = external_pipeline.get_python_origin()
+        pipeline_origin = context.pipeline_code_origin
         repository_origin = pipeline_origin.repository_origin
 
         job_config = (
@@ -280,7 +329,6 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             ),
             cls=self.__class__,
         )
-        return run
 
     # https://github.com/dagster-io/dagster/issues/2741
     def can_terminate(self, run_id):
@@ -303,9 +351,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         can_terminate = self.can_terminate(run_id)
         if not can_terminate:
             self._instance.report_engine_event(
-                message="Unable to terminate pipeline; can_terminate returned {}".format(
-                    can_terminate
-                ),
+                message="Unable to terminate run; can_terminate returned {}".format(can_terminate),
                 pipeline_run=run,
                 cls=self.__class__,
             )
@@ -319,13 +365,13 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             termination_result = delete_job(job_name=job_name, namespace=self.job_namespace)
             if termination_result:
                 self._instance.report_engine_event(
-                    message="Pipeline was terminated successfully.",
+                    message="Run was terminated successfully.",
                     pipeline_run=run,
                     cls=self.__class__,
                 )
             else:
                 self._instance.report_engine_event(
-                    message="Pipeline was not terminated successfully; delete_job returned {}".format(
+                    message="Run was not terminated successfully; delete_job returned {}".format(
                         termination_result
                     ),
                     pipeline_run=run,
@@ -334,7 +380,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             return termination_result
         except Exception:  # pylint: disable=broad-except
             self._instance.report_engine_event(
-                message="Pipeline was not terminated successfully; encountered error in delete_job",
+                message="Run was not terminated successfully; encountered error in delete_job",
                 pipeline_run=run,
                 engine_event_data=EngineEventData.engine_error(
                     serializable_error_info_from_exc_info(sys.exc_info())
